# coding: utf-8
from itertools import imap
import abc

from xlsxwriter import Workbook

from educommon.utils.misc import cached_property

from ...exceptions import DataSourceNotFound
from ...models import ReportTemplate
from ...registries import registry


class ReportBuilderBase(object):
    u"""Базовый класс построителя отчета."""

    __metaclass__ = abc.ABCMeta

    def __init__(self, template, file_path):
        u"""Инициализация класса.

        :param template: Шаблон отчета.
        :type template: educommon.report.constructor.models.ReportTemplate

        :param unicode file_path: Путь к файлу отчета.
        """

        assert isinstance(template, ReportTemplate), type(
            template
        )

        if template.data_source_name not in registry:
            raise DataSourceNotFound(template.data_source_name)

        self._template = template
        self._file_path = file_path

    @cached_property
    def _data_source(self):
        return self._template.data_source

    @cached_property
    def _report_columns(self):
        return tuple(
            self._template.columns.all()
        )

    @cached_property
    def _data(self):
        return self._data_source.get_data(*(
            imap(lambda col: col.name, self._report_columns)
        ))

    @cached_property
    def _workbook(self):
        result = Workbook(self._file_path, dict(
            default_date_format='dd.mm.yyyy',
        ))

        for cell_format in (result.default_date_format,
                            result.default_url_format):
            cell_format.set_border()
            cell_format.set_align('vcenter')

        return result

    @cached_property
    def _worksheet(self):
        return self._workbook.add_worksheet(u'Отчет')

    @abc.abstractmethod
    def _flush_header(self):
        u"""Запись заголовка отчета."""

    @abc.abstractmethod
    def _flush_data(self):
        u"""Запись данных в отчет."""

    def build(self):
        u"""Формирование отчета."""
        self._flush_header()
        self._flush_data()

        self._workbook.close()
